@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: limglib.c 3404 2010-01-28 11:17:10Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/lua/limglib.c $";
+    "$Id: limglib.c 3676 2010-05-02 20:04:49Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.63.0/source/texk/web2c/luatexdir/lua/limglib.c $";
 
 #include <stdio.h>
 #include <string.h>
@@ -424,7 +424,7 @@ static void lua_to_image(lua_State * L, image * a)
 
 /**********************************************************************/
 
-void copy_image(lua_State * L, lua_Number scale)
+static void copy_image(lua_State * L, lua_Number scale)
 {
     image *a, **aa, *b, **bb;
     if (lua_gettop(L) != 1)
@@ -609,6 +609,8 @@ static int l_write_image(lua_State * L)
 static int l_immediatewrite_image(lua_State * L)
 {
     check_o_mode(static_pdf, "img.immediatewrite()", 1 << OMODE_PDF, true);
+    if (global_shipping_mode != NOT_SHIPPING)
+        luaL_error(L, "pdf.immediatewrite() can not be used with \\latelua");
     write_image_or_node(L, WR_IMMEDIATEWRITE);
     return 1;                   /* image */
 }
