@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: lnodelib.c 3560 2010-03-29 09:27:08Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/lua/lnodelib.c $";
+    "$Id: lnodelib.c 3851 2010-09-03 11:55:16Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.63.0/source/texk/web2c/luatexdir/lua/lnodelib.c $";
 
 #include "lua/luatex-api.h"
 #include "ptexlib.h"
@@ -685,7 +685,7 @@ make_luaS_index(depth);
 make_luaS_index(expansion_factor);
 
 
-void initialize_luaS_indexes(lua_State * L)
+static void initialize_luaS_indexes(lua_State * L)
 {
     init_luaS_index(id);
     init_luaS_index(next);
@@ -1374,15 +1374,20 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
             break;
         case late_lua_node:
             switch (field) {
-            case 4:
+            case 4: /* regid (obsolete?)*/
                 lua_pushnumber(L, late_lua_reg(n));
                 break;
-            case 5:
-                tokenlist_to_luastring(L, late_lua_data(n));
-                break;
-            case 6:
+            case 6: /* name */
                 tokenlist_to_luastring(L, late_lua_name(n));
                 break;
+            case 5: /* data */
+            case 7: /* string */
+                if (late_lua_type(n) == lua_refid_literal) {
+                    lua_rawgeti(Luas, LUA_REGISTRYINDEX, late_lua_data(n));
+                } else {
+                    tokenlist_to_luastring(L, late_lua_data(n));
+                }
+                break;
             default:
                 lua_pushnil(L);
             }
@@ -2163,7 +2168,8 @@ static void nodelib_setattr(lua_State * L, int stackindex, halfword n)
         if (node_attr(n) != null)
             delete_attribute_ref(node_attr(n));
         node_attr(n) = p;
-        attr_list_ref(p)++;
+	if (p != null)
+	    attr_list_ref(p)++;
     }
 }
 
@@ -2457,12 +2463,23 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
         case 4:
             late_lua_reg(n) = (halfword) lua_tointeger(L, 3);
             break;
-        case 5:
+        case 5: /* data */
             late_lua_data(n) = nodelib_gettoks(L, 3);
+            late_lua_type(n) = normal;
             break;
         case 6:
             late_lua_name(n) = nodelib_gettoks(L, 3);
             break;
+        case 7: /* string */
+            if (ini_version) {
+                late_lua_data(n) = nodelib_gettoks(L, 3);
+                late_lua_type(n) = normal;
+            } else {
+                lua_pushvalue(L, 3);
+                late_lua_data(n) = luaL_ref(L, LUA_REGISTRYINDEX);
+                late_lua_type(n) = lua_refid_literal;
+            }
+            break;
         default:
             return nodelib_cantset(L, field, n);
         }
@@ -3308,7 +3325,7 @@ static int lua_nodelib_first_character(lua_State * L)
 
 /* this is too simplistic, but it helps Hans to get going */
 
-halfword do_ligature_n(halfword prev, halfword stop, halfword lig)
+static halfword do_ligature_n(halfword prev, halfword stop, halfword lig)
 {
     vlink(lig) = vlink(stop);
     vlink(stop) = null;
@@ -3395,6 +3412,7 @@ static const struct luaL_reg nodelib_f[] = {
     {"protect_glyphs", lua_nodelib_protect_glyphs},
     {"unprotect_glyphs", lua_nodelib_unprotect_glyphs},
     {"protrusion_skipable", lua_nodelib_cp_skipable},
+    {"protrusion_skippable", lua_nodelib_cp_skipable},
    {NULL, NULL}                /* sentinel */
 };
 
