@@ -28,10 +28,15 @@
 #include <ustring.h>
 
 static const char _svn_version[] =
-    "$Id: ffdummies.c 3360 2010-01-12 21:12:15Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/luafontloader/src/ffdummies.c $";
+    "$Id: ffdummies.c 3738 2010-07-13 10:28:24Z oneiros $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.63.0/source/texk/web2c/luatexdir/luafontloader/src/ffdummies.c $";
 
 
+/* fix for non-gcc compilation: */
+#if !defined(__GNUC__) || (__GNUC__ < 2)
+#  define __attribute__(x)
+#endif                          /* !defined(__GNUC__) || (__GNUC__ < 2) */
+
 char **gww_errors = NULL;
 int gww_error_count = 0;
 
@@ -57,7 +62,7 @@ static void LUAUI_IError(const char *format, ...)
     l = strlen("Internal Error: ");
     snprintf(buffer, sizeof(buffer), "Internal Error: ");
     va_start(ap, format);
-    vsnprintf(buffer+l, sizeof(buffer)-l, format, ap);
+    vsnprintf(buffer + l, sizeof(buffer) - l, format, ap);
     va_end(ap);
     str = xstrdup((char *) buffer);
     gww_errors = realloc(gww_errors, (gww_error_count + 2) * sizeof(char *));
