@@ -10,30 +10,33 @@
    #defines TeX or MF, which avoids the need for a special
    Makefile rule.  */
 #include "luatex.h"
-
+#include "ptexlib.h"
+#include "lua/luatex-api.h"
 #include "luatex_svnversion.h"
 
 static const char _svn_version[] =
-    "$Id: luatex.c 3588 2010-04-04 06:41:44Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.60.0/source/texk/web2c/luatexdir/luatex.c $";
+    "$Id: luatex.c 3826 2010-08-23 18:57:30Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.63.0/source/texk/web2c/luatexdir/luatex.c $";
 
 #define TeX
 
 int luatex_svn = luatex_svn_revision;
-int luatex_version = 60;        /* \.{\\luatexversion}  */
+int luatex_version = 63;        /* \.{\\luatexversion}  */
 int luatex_revision = '0';      /* \.{\\luatexrevision}  */
 int luatex_date_info = -extra_version_info;     /* the compile date is negated */
-const char *luatex_version_string = "beta-0.60.0";
+const char *luatex_version_string = "beta-0.63.0";
 const char *engine_name = "luatex";     /* the name of this engine */
 
-
-
 #include <kpathsea/c-ctype.h>
 #include <kpathsea/line.h>
 #include <kpathsea/readable.h>
 #include <kpathsea/variable.h>
 #include <kpathsea/absolute.h>
 #include <kpathsea/recorder.h>
+#ifdef WIN32
+#include <kpathsea/concatn.h>
+#endif
+
 
 #include <time.h>               /* For `struct tm'.  */
 #if defined (HAVE_SYS_TIME_H)
@@ -48,7 +51,6 @@ const char *engine_name = "luatex";     /* the name of this engine */
 
 #include <signal.h>             /* Catch interrupts.  */
 
-#include <texmfmp-help.h>
 
 /* {tex,mf}d.h defines TeX, MF, INI, and other such symbols.
    Unfortunately there's no way to get the banner into this code, so
@@ -349,6 +351,36 @@ int shell_cmd_is_allowed(const char **cmd, char **safecmd, char **cmdname)
             *d++ = QUOTE;
         }
         *d = '\0';
+#ifdef WIN32
+        {
+          char *p, *q, *r;
+          p = *safecmd;
+          if (!(IS_DIR_SEP (p[0]) && IS_DIR_SEP (p[1])) &&
+              !(p[1] == ':' && IS_DIR_SEP (p[2]))) { 
+            p = (char *) kpse_var_value ("SELFAUTOLOC");
+            if (p) {
+              r = *safecmd;
+              while (*r && !Isspace(*r))
+                r++;
+              if (*r == '\0')
+                q = concatn ("\"", p, "/", *safecmd, "\"", NULL);
+              else {
+                *r = '\0';
+                r++;
+                while (*r && Isspace(*r))
+                  r++;
+                if (*r)
+                  q = concatn ("\"", p, "/", *safecmd, "\" ", r, NULL);
+                else
+                  q = concatn ("\"", p, "/", *safecmd, "\"", NULL);
+              }
+              free (p);
+              free (*safecmd);
+              *safecmd = q;
+            }
+          }
+        }
+#endif
     }
 
     return allow;
@@ -395,9 +427,6 @@ int runsystem(char *cmd)
 
 #endif
 
-/* The main program, etc.  */
-
-extern void lua_initialize(int ac, char **av);
 
 /* What we were invoked as and with.  */
 char **argv;
@@ -409,13 +438,7 @@ string dump_name;
 /* The C version of the jobname, if given. */
 const_string c_job_name;
 
-/* Full source file name. */
-extern string fullnameoffile;
-
-/* The main body of the WEB is transformed into this procedure.  */
-extern TEXDLL void mainbody(void);
-
-char *ptexbanner;
+const char *ptexbanner;
 
 #if !defined(WIN32) || defined(__MINGW32__)
 /* The entry point: set up for reading the command line, which will
@@ -435,7 +458,7 @@ int main(int ac, string * av)
     lua_initialize(ac, av);
 
     /* Call the real main program.  */
-    mainbody();
+    main_body();
 
     return EXIT_SUCCESS;
 }
@@ -488,6 +511,9 @@ void topenin(void)
    incrementally.  Shamim Mohamed adapted it for Web2c.  */
 #if defined (TeX) && defined (IPC)
 
+# ifdef WIN32
+#  include <winsock2.h>
+# else
 #  include <sys/socket.h>
 #  include <fcntl.h>
 #  ifndef O_NONBLOCK            /* POSIX */
@@ -501,6 +527,8 @@ what the fcntl ? cannot implement IPC without equivalent for O_NONBLOCK.
 #      endif                    /* no FNDELAY */
 #    endif                      /* no O_NDELAY */
 #  endif                        /* no O_NONBLOCK */
+# endif /* !WIN32 */
+
 #  ifndef IPC_PIPE_NAME         /* $HOME is prepended to this.  */
 #    define IPC_PIPE_NAME "/.TeXview_Pipe"
 #  endif
@@ -546,6 +574,12 @@ static int ipc_is_open(void)
 
 static void ipc_open_out(void)
 {
+#ifdef WIN32
+  u_long mode = 1;
+#define SOCK_NONBLOCK(s) ioctlsocket (s, FIONBIO, &mode)
+#else
+#define SOCK_NONBLOCK(s) fcntl (s, F_SETFL, O_NONBLOCK)
+#endif
 #  ifdef IPC_DEBUG
     fputs("tex: Opening socket for IPC output ...\n", stderr);
 #  endif
@@ -561,7 +595,7 @@ static void ipc_open_out(void)
     sock = socket(PF_UNIX, SOCK_STREAM, 0);
     if (sock >= 0) {
         if (connect(sock, ipc_addr, ipc_addr_len) != 0
-            || fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
+            || SOCK_NONBLOCK (sock) < 0) {
             close(sock);
             sock = -1;
             return;
@@ -626,42 +660,26 @@ void ipcpage(int is_eof)
 {
     static boolean begun = false;
     unsigned len = 0;
-    string p = (string) "";
+    string p = NULL;
 
     if (!begun) {
         string name;            /* Just the filename.  */
         string cwd = xgetcwd();
 
         ipc_open_out();
-#  if !defined(Aleph)
-        len = strstart[outputfilename + 1] - strstart[outputfilename];
-#  else
-        len = strstartar[outputfilename + 1 - 65536L] -
-            strstartar[outputfilename - 65536L];
-#  endif
-        name = (string) xmalloc(len + 1);
-#  if !defined(Aleph)
-        strncpy(name, (string) & strpool[strstart[outputfilename]], len);
-#  else
-        {
-            unsigned i;
-            for (i = 0; i < len; i++)
-                name[i] = strpool[i + strstartar[outputfilename - 65536L]];
-        }
-#  endif
-        name[len] = 0;
 
         /* Have to pass whole filename to the other end, since it may have
            been started up and running as a daemon, e.g., as with the NeXT
            preview program.  */
+	name = static_pdf->file_name;
         p = concat3(cwd, DIR_SEP_STRING, name);
-        free(name);
+        /* free(name); */
         len = strlen(p);
         begun = true;
     }
     ipc_snd(len, is_eof, p);
 
-    if (len > 0) {
+    if (p) {
         free(p);
     }
 }
@@ -701,7 +719,7 @@ string normalize_quotes(const_string name, const_string mesg)
    variable `interrupt'; then they will do everything needed.  */
 #ifdef WIN32
 /* Win32 doesn't set SIGINT ... */
-BOOL WINAPI catch_interrupt(DWORD arg)
+static BOOL WINAPI catch_interrupt(DWORD arg)
 {
     switch (arg) {
     case CTRL_C_EVENT:
@@ -798,7 +816,7 @@ void get_seconds_and_micros(int *seconds, int *micros)
 /*
   Generating a better seed numbers
   */
-int getrandomseed()
+int getrandomseed(void)
 {
 #if defined (HAVE_GETTIMEOFDAY)
     struct timeval tv;
@@ -863,13 +881,16 @@ boolean input_line(FILE * f)
 
 /* This string specifies what the `e' option does in response to an
    error message.  */
+#if 0
 static const_string edit_value = EDITOR;
+#endif
 
 /* This procedure originally due to sjc@s1-c.  TeX & Metafont call it when
    the user types `e' in response to an error, invoking a text editor on
    the erroneous source file.  FNSTART is how far into FILENAME the
    actual filename starts; FNLENGTH is how long the filename is.  */
 
+#if 0
 void
 calledit(packedASCIIcode * filename,
          poolpointer fnstart, int fnlength, int linenumber)
@@ -938,7 +959,7 @@ calledit(packedASCIIcode * filename,
     *temp = 0;
 
     /* Execute the command.  */
-#ifdef WIN32
+#ifdef __MINGW32__
     /* Win32 reimplementation of the system() command
        provides opportunity to call it asynchronously */
     if (win32_system(command, true) != 0)
@@ -950,6 +971,7 @@ calledit(packedASCIIcode * filename,
     /* Quit, since we found an error.  */
     uexit(1);
 }
+#endif
 
 /* Read and write dump files.  As distributed, these files are
    architecture dependent; specifically, BigEndian and LittleEndian
@@ -964,16 +986,19 @@ calledit(packedASCIIcode * filename,
 /* This macro is always invoked as a statement.  It assumes a variable
    `temp'.  */
 
-#  define SWAP(x, y) temp = (x); (x) = (y); (y) = temp
+#  define SWAP(x, y) do { temp = x; x = y; y = temp; } while (0)
 
 
 /* Make the NITEMS items pointed at by P, each of size SIZE, be the
    opposite-endianness of whatever they are now.  */
 
-static void swap_items(char *p, int nitems, int size)
+void swap_items(char *pp, int nitems, int size)
 {
     char temp;
-
+    unsigned total = nitems*size;
+    char *q = xmalloc(total);
+    char *p = q;
+    memcpy(p,pp,total);
     /* Since `size' does not change, we can write a while loop for each
        case, and avoid testing `size' for each time.  */
     switch (size) {
@@ -993,6 +1018,18 @@ static void swap_items(char *p, int nitems, int size)
         }
         break;
 
+    case 12:
+        while (nitems--) {
+            SWAP(p[0], p[11]);
+            SWAP(p[1], p[10]);
+            SWAP(p[2], p[9]);
+            SWAP(p[3], p[8]);
+            SWAP(p[4], p[7]);
+            SWAP(p[5], p[6]);
+            p += size;
+        }
+        break;
+
     case 8:
         while (nitems--) {
             SWAP(p[0], p[7]);
@@ -1025,10 +1062,13 @@ static void swap_items(char *p, int nitems, int size)
     default:
         FATAL1("Can't swap a %d-byte item for (un)dumping", size);
     }
+    memcpy(pp,q,total);
+    xfree(q); 
 }
 #endif                          /* not WORDS_BIGENDIAN and not NO_DUMP_SHARE */
 
 
+#if 0
 /* Here we write NITEMS items, each item being ITEM_SIZE bytes long.
    The pointer to the stuff to write is P, and we write to the file
    OUT_FILE.  */
@@ -1052,10 +1092,12 @@ void do_dump(char *p, int item_size, int nitems, FILE * out_file)
     swap_items(p, nitems, item_size);
 #endif
 }
+#endif
+
 
 
 /* Here is the dual of the writing routine.  */
-
+#if 0
 void do_undump(char *p, int item_size, int nitems, FILE * in_file)
 {
     if (fread(p, (size_t) item_size, (size_t) nitems, in_file) !=
@@ -1066,6 +1108,7 @@ void do_undump(char *p, int item_size, int nitems, FILE * in_file)
     swap_items(p, nitems, item_size);
 #endif
 }
+#endif
 
 
 /* Get the job name to be used, which may have been set from the
